/**
********************************************************************************
Ortus DataBoss
Copyright 2012 Ortus Solutions, Corp
www.ortussolutions.com
********************************************************************************
* DataBoss Module Configuration
*/
component{

	// Module Type
	projectType = "Multi-Project";

	// Module Properties
	this.title 					= "Ortus DataBoss #projectType#";
	this.author 				= "Ortus Solutions, Corp";
	this.webURL 				= "http://www.ortussolutions.com";
	this.description 			= "Dynamic Administrator: Manage CFML ORM entities the DataBoss Way!";
	this.version				= "1.4.0.@build.number@";
	this.viewParentLookup 		= true;
	this.layoutParentLookup		= true;
	this.entryPoint				= "databoss";

	/*
	* Configure the Module
	*/
	function configure(){

		// Store SBEK
		variables._s = "knAd0IF3wHHZVYPebRoP1Q==";
		variables._activated = false;
		// dev mode
		variables._inDevMode = true;
		// Trial Setup
		variables._trialMode = false;
		variables._trialExpireDate = "@EXPIRE_DATE@";
		// config path
		variables.configPath = modulePath & "/config/Databoss.json.cfm";

		// ****************************************************************************************** //
		// DataBoss Settings (may be overridden by Databoss.json config)
		// ****************************************************************************************** //
		settings = {
			// License Key holder
			licenseKey = "",
			// License email address
			licenseEmail = "",
			// DataBoss Version
			version 			= this.version,
			// Pagination
			pagingMaxRows 		= "20",
			pagingBandGap 		= "5",
			// Load jQuery?
			loadJquery 			= true,
			// Localization Settings
			showLanguageOptions = true,
			supportedLanguages 	= ['en_US', 'es_CO', 'de_DE', 'fr_FR', 'pt_BR', 'it_IT'],
			defaultLocale		= "en_US",
			// Listing Max Chars
			listingMaxChars		= 100,
			// Show Dictionary Dump
			debugMode 			= false,
			// Show Global Actions
			showGlobalActions	= true,
			// Show Logo
			showLogo			= true,
			// Show About
			showAbout			= true,
			// JSONP CallBack
			jsonpcallback		= "",
			// Basic Auth
			basicAuthentication = {
				enabled = false,
				username = "admin",
				password = "databoss"
			},
			// Logging
			logging = {
				enabled = true, levelMin="FATAL", levelMax="INFO"
			}
		};

		// Load custom settings from disk
		loadSettings();

		// Date Set Trial
		if( variables._trialMode and dateCompare( dateFormat( now(), "mm-dd-yyyy" ), variables._trialExpireDate ) eq 1 ){
			writeoutput("Ortus DataBoss Trial expired. Please contact us at info@ortussolutions.com for more information.");
			abort;
		}

		// Verify License, else don't load
		var licenseResults = verifyLicense();
		variables._activated = licenseResults.valid;

		// Mark Loaded or Unloaded
		if( !variables._trialMode && !variables._activated ){
			// Inform about license issue
			writeOutput( licenseResults.messages );
			writeoutput("<br>Please contact us at <a href='mailto:info@ortussolutions.com'>info@ortussolutions.com</a> with your license information.");
			abort;
		}

		// Layouts
		layoutSettings = {
			defaultLayout = "DataBoss.cfm"
		};

		// SES DataBoss Routes
		routes = [
			{ pattern="/entity/:entity/id/:id", handler="databoss",action="editor" },
			{ pattern="/entity/:entity/page/:page", handler="databoss" },
			{ pattern="/entity/:entity/export/:format", handler="databoss", action="index" },
			{ pattern="/entity/:entity/showAll", handler="databoss", showAll=true},
			{ pattern="/entity/:entity/:action", handler="databoss"} ,
			{ pattern="/entity/:entity", handler="databoss", action="index" },
			{ pattern="/:action", handler="databoss" },
			{ pattern="/", handler="databoss",action="index" }
		];

		// WireBox Mappings
		binder.map("entityDisplayHelper@databoss")
			.to("#moduleMapping#.model.entityDisplayHelper");

		binder.map("metadataService@databoss")
			.to("#moduleMapping#.model.MetadataService");

		binder.map("entityService@databoss")
			.to("#moduleMapping#.model.EntityService");

		binder.map("paging@databoss")
			.toDSL("coldbox:myplugin:Paging@databoss");

		binder.map("securityService@databoss")
			.to("#moduleMapping#.model.security.SecurityService");

		// Interceptor Events
		interceptorSettings = {
			// ContentBox UI Custom Events, you can add your own if you like to!
			customInterceptionPoints = arrayToList([
				// UI Events
				"db_header_logo", "db_leftnavbar_start", "db_leftnavbar_end", "db_rightnavbar_start", "db_rightnavbar_end",
				"db_afterBodyStart", "db_afterBodyEnd", "db_afterHeadStart", "db_afterHeadEnd"
			])
		};

		// Interceptors
		interceptors = [
			{ name="databoss@basicAuth", class="#moduleMapping#.model.security.BasicAuth" }
		];

	}

	/**
	* Executes when the module loads
	*/
	function onLoad(){
		// don't execute if not activated
		if( !variables._trialMode && !variables._activated ){ return false; }

		// get settings for this particular module
		var moduleSettings = controller.getConfigSettings().modules.databoss.settings;

		// Add DataBoss Logging?
		if( moduleSettings.logging.enabled ){
			// Add databoss apender
			var logBoxConfig = controller.getLogBox().getConfig();
			var appenderProps = { filepath="#modulePath#/logs", filename="databoss", autoExpand=false };
			logBoxConfig.appender(name="databoss_logger",
								  class="coldbox.system.logging.appenders.AsyncRollingFileAppender",
								  properties=appenderProps,
								  levelMin=moduleSettings.logging.levelMin,
								  levelMax=moduleSettings.logging.levelMax);
			// Add databoss category
			logBoxConfig.category(name="databoss",
								  appenders="databoss_logger",
								  levelMin=moduleSettings.logging.levelMin,
								  levelMax=moduleSettings.logging.levelMax);
			// Store back config
			controller.getLogBox().configure( logBoxConfig );
		}

		// add localization settings
		if( controller.getSetting( 'using_i18n' ) ){
			// parent app already has i18n configured
			controller.getPlugin( 'ResourceBundle' )
				.loadBundle(rbFile='#moduleMapping#/includes/i18n/main', rbLocale='en_US');
		}
		else{
			// parent app does not have i18n configured, so add settings manually
			controller.setSetting( 'LocaleStorage', 'cookie' );
			controller.setSetting( 'defaultResourceBundle', moduleMapping & '/includes/i18n/main' );
			controller.setSetting( 'defaultLocale', settings.defaultLocale );
			// initialize i18n plugin
			controller.getPlugin( "i18n" )
				.init_i18N( controller.getSetting( "DefaultResourceBundle" ), controller.getSetting( "DefaultLocale" ) );
		}

	}

	/**
	* Executes when the process finalizes
	*/
	function postProcess(event, interceptData){
		// do restarts if sent.
		if( event.getCurrentEvent() eq "databoss:databoss.reloadApp"){
			applicationStop();
		}
	}

	/************************************** PRIVATE *********************************************/

	// validate license
	private function verifyLicense(){
		var results = { valid = false, messages = "" };

		// dev mode?
		if( variables._inDevMode ){ results.valid = true; return results; }

		// check length key
		if( !len( settings.licenseKey ) ){
			results.messages = "The license key is empty, cannot continue operation";
			return results;
		}
		// check length email
		if( !len( settings.licenseEmail ) ){
			results.messages = "The license email is empty, cannot continue operation";
			return results;
		}

		// Check decryption
		try{
			var data = decrypt( settings.licenseKey, variables._s, "BLOWFISH", "Base64" );
			// check JSON
			if( isJSON( data ) ){
				// deserialize it
				data = deserializeJSON( data );
				// Check the licenseEmail values
				if( settings.licenseEmail eq data.email ){
					// Check if we have a perpetual license
					if( structKeyExists( data, "licenseType" ) and data.licenseType eq "perpetual" ){
						results.valid = true;
					}
					// Finally check the date for 1 year validity for subscription
					else if( dateDiff( "yyyy", data.orderdate, now() ) eq 0 ){
						results.valid = true;
					}
					// else it expired
					else {
						results.messages = "The current license validity period has expired (#data.orderDate#). Error Code: 00EE1";
					}
				} else {
					results.messages = "The license email information is not valid, cannot continue. Error Code: 00E1";
				}
			} else {
				results.messages = "The license information is not valid, cannot continue. Error Code: 00J1";
			}
		} catch(Any e){
			results.messages = "The license information is not valid, cannot continue. Error Code: 00D1";
		}

		return results;
	}

	// Load configuration settings
	private function loadSettings(){
		// get settings for this particular module
		//var moduleSettings = controller.getConfigSettings().modules[ 'fr-profilebox' ].settings;
		// check for JSON-based config overrides

		if( fileExists( configPath ) ){
			// load JSON file and deserialize it
			var settingOverrides = deserializeJSON( fileRead( configPath ) );
			// loop through JSON settings and override module settings
			for( var thisSetting in settingOverrides ){
				// validate setting exists
				if( structKeyExists( variables.settings, thisSetting ) ){
					variables.settings[ thisSetting ] = settingOverrides[ thisSetting ];
				}
			}
		}
	}

}