/**
********************************************************************************
Copyright 2012 Ortus Solutions, Corp
www.ortussolutions.com
********************************************************************************

Author     :	Luis Majano
Description :
	This is the standalone Application.cfc for Ortus Databoss if you will be
	running it as a standalone application and NOT a ColdBox module.

	Modify the ORM settings in order for the ORM to look at your entities, configuration
	and datasource. DataBoss relies on your ORM configuration.
*/
component{

	// Application CFC Properties: Change as you see fit
	this.name = "Ortus DataBoss - " & hash(getCurrentTemplatePath());
	this.sessionManagement = true;
	this.sessionTimeout = createTimeSpan(0,0,30,0);
	this.setClientCookies = true;

	// ColdBox Settings
	COLDBOX_APP_ROOT_PATH = getDirectoryFromPath( getCurrentTemplatePath() );
	COLDBOX_APP_MAPPING		= "";
	COLDBOX_CONFIG_FILE 	= "";
	COLDBOX_APP_KEY 		= "";
	// THE LOCATION OF EMBEDDED COLDBOX
	this.mappings["/coldbox"] 	 = COLDBOX_APP_ROOT_PATH & "coldbox";
	// DATABOSS LOCALIZED MAPPING
	this.mappings["/DataBoss"] = COLDBOX_APP_ROOT_PATH;

	// Mapping Imports
	import coldbox.system.*;

	// *************************************************************************************** //
	// ORM SETTINGS: CONFIGURE TO YOUR LIKING
	// *************************************************************************************** //
	this.ormEnabled 	  = true;
	// CHOOSE THE DATASOURCE YOU WANT DATABOSS TO MANAGE
	this.datasource		  = "databoss";
	this.ormSettings	  = {
		// THIS IS WHERE YOUR ORM ENTITIES ARE LOCATED, FILL OUT TO YOUR LIKING
		cfclocation=[ "model" ],
		dbcreate 	= "update",
		//dialect 	= "MySQLwithInnoDB",
		// ORM SESSION MANAGEMENT SETTINGS, PLEASE DO NOT ALTER
		logSQL 				= true,
		flushAtRequestEnd 	= false,
		autoManageSession	= false,
		// ORM EVENTS, PLEASE DO NOT ALTER
		eventHandling 		= true,
		eventHandler		= "coldbox.system.orm.hibernate.WBEVentHandler"
	};
	// *************************************************************************************** //
	// END ORM SETTINGS
	// *************************************************************************************** //

	public boolean function onApplicationStart(){
		application.cbBootstrap = new Coldbox(COLDBOX_CONFIG_FILE,COLDBOX_APP_ROOT_PATH,COLDBOX_APP_KEY);
		application.cbBootstrap.loadColdbox();
		return true;
	}

	public boolean function onRequestStart(String targetPage){

		//if( structKeyExists(url,"ormReload") ){ ormReload(); }
		//applicationstop();abort;

		// Bootstrap Reinit
		if( not structKeyExists(application,"cbBootstrap") or application.cbBootStrap.isfwReinit() ){
			lock name="coldbox.bootstrap_#this.name#" type="exclusive" timeout="5" throwonTimeout=true{
				structDelete(application,"cbBootStrap");
				application.cbBootstrap = new ColdBox(COLDBOX_CONFIG_FILE,COLDBOX_APP_ROOT_PATH,COLDBOX_APP_KEY,COLDBOX_APP_MAPPING);
			}
		}

		// ColdBox Reload Checks
		application.cbBootStrap.reloadChecks();

		//Process a ColdBox request only
		if( findNoCase('index.cfm',listLast(arguments.targetPage,"/")) ){
			application.cbBootStrap.processColdBoxRequest();
		}

		return true;
	}

	public void function onSessionStart(){
		application.cbBootStrap.onSessionStart();
	}

	public void function onSessionEnd(struct sessionScope, struct appScope){
		arguments.appScope.cbBootStrap.onSessionEnd(argumentCollection=arguments);
	}

	public boolean function onMissingTemplate(template){
		return application.cbBootstrap.onMissingTemplate(argumentCollection=arguments);
	}

}